<?php

namespace Lean\Api;

use League\CommonMark\CommonMarkConverter;
use Notes\Attribute\Ignore;
use Psr\Http\Message\ResponseInterface;

trait LeanApiTrait
{
    use DescriptorTrait;

    #[Ignore]
    public function renderMarkdown(string $filepath, array $entities = [], array $forms = [], int $code = 200, array $headers = []) : ResponseInterface
    {
        if ( ! class_exists(CommonMarkConverter::class)) {
            throw new \BadFunctionCallException("League\CommonMark seems to be missing, please install dependency before trying to render Markdown content");
        }

        $markdown = ( new CommonMarkConverter() )->convert(file_get_contents($filepath))->getContent();

        if (str_contains($markdown, '{route:descriptor}'))
        {
            $markdown = str_replace('{route:descriptor}', $this->renderRawView('lean-api/route_descriptor', [
                'routes' => (new RouteDescriptor($this, $this->picea->compiler->getExtensionFromToken('url')))->getRoutes()
            ]), $markdown);
        }

        if (str_contains($markdown, '{entity:descriptor}'))
        {
            $markdown = str_replace('{entity:descriptor}', $this->renderRawView('lean-api/entity_descriptor', [
                'entities' => (new EntityDescriptor())->getEntities(... $entities)
            ]), $markdown);
        }

        if (str_contains($markdown, '{form:descriptor}'))
        {
            $markdown = str_replace('{form:descriptor}', $this->renderRawView('lean-api/form_descriptor', [
                'forms' => (new FormDescriptor())->getForms(... $forms)
            ]), $markdown);
        }

        return $this->renderView("lean/layout/docs", get_defined_vars());
    }

    #[Ignore]
    protected function output(\JsonSerializable|array $data, int $count) {
        return $this->renderJson([
            'data' => $data,
            'count' => $count,
        ]);
    }
}